#!/bin/bash

# test is we are running on Ubuntu
if [ $(uname -v | cut -d '-' -f 2 | awk '{print $1}') != "Ubuntu" ];
then
  echo
  echo This script currently only works on Ubuntu
  echo
  exit
fi

if [ ! -f ./nginx.conf  ]; then
    echo "One of the configuration files are missing"
    exit 1
fi

if [ ! -f ./default.conf  ]; then
    echo "One of the configuration files are missing"
    exit 1
fi

if [ ! -f ./serverblock.conf  ]; then
    echo "One of the configuration files are missing"
    exit 1
fi

if [ $# -eq 0 ]
  then
    echo
    echo $(tput setaf 1)
    echo This script will install Modx under Openresty
    echo If Openresty can\'t be found then it will be installed first
    echo
    echo It needs SUDO privileges so will ask for them where necessary
    echo
    echo "Usage: ./InstallModX.sh domain_name (without http://, https:// or www.)"
    echo
    echo "Note: Make sure that this domain name is already pointing to the server"
    echo "      and a CNAME is set up for www"
    echo
    echo "Your site will be available at https://www.[domain_name]"
    echo
    echo $(tput setaf 7)
    exit 1
fi

sitename=$1

if [[ "$sitename" == *"http://"* ]]; then
  echo "Usage: ./InstallModX.sh domain_name (without http\:\/\/, https\:\/\/ or www\.)"
  exit 1
fi

if [[ "$sitename" == *"https://"* ]]; then
  echo "Usage: ./InstallModX.sh domain_name (without http\:\/\/, https\:\/\/ or www\.)"
  exit 1
fi

if [[ "$sitename" == *"www."* ]]; then
  echo "Usage: ./InstallModX.sh domain_name (without http\:\/\/, https\:\/\/ or www\.)"
  exit 1
fi

if [[ "$sitename" == *"HTTP://"* ]]; then
  echo "Usage: ./InstallModX.sh domain_name (without http\:\/\/, https\:\/\/ or www\.)"
  exit 1
fi

if [[ "$sitename" == *"HTTPS://"* ]]; then
  echo "Usage: ./InstallModX.sh domain_name (without http\:\/\/, https\:\/\/ or www\.)"
  exit 1
fi

if [[ "$sitename" == *"WWW."* ]]; then
  echo "Usage: ./InstallModX.sh domain_name (without http\:\/\/, https\:\/\/ or www\.)"
  exit 1
fi

# Install some needed utilities if not already installed
sudo apt install lsb-release ca-certificates apt-transport-https software-properties-common language-pack-en-base unzip -y

# Install PHP
phpver=8.1

sudo add-apt-repository ppa:ondrej/php -y
sudo apt update
sudo apt install php$phpver-{common,cli,bz2,zip,curl,intl,mysql,snmp,memcached,imagick,gd,imap,ldap,soap,tidy,xml,gmp,pspell,mbstring,opcache,fpm,ssh2,imap,redis,apcu,mcrypt} -y
sudo apt install ghostscript imagemagick -y

# Configure PHP
sudo sed -i -e '/^\(;\|\)listen.mode = 0660/s/^.*$/listen.mode = 0660/' /etc/php/$phpver/fpm/pool.d/www.conf
sudo sed -i -e '/^\(;\|\)output_buffering = 4096/s/^.*$/output_buffering = Off/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)zlib.output_compression = Off/s/^.*$/zlib.output_compression = On/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)zlib.output_compression_level = -1/s/^.*$/zlib.output_compression_level = 5/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)expose_php =/s/^.*$/expose_php = Off/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)max_execution_time =/s/^.*$/max_execution_time = 60/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)max_input_time =/s/^.*$/max_input_time = 60/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)max_input_vars =/s/^.*$/max_input_vars = 1000/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)memory_limit =/s/^.*$/memory_limit = 512M/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)post_max_size =/s/^.*$/post_max_size = 8M/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)upload_max_filesize =/s/^.*$/upload_max_filesize = 4M/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)allow_url_fopen =/s/^.*$/allow_url_fopen = Off/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\;extension=fileinfo/s/^.*$/extension=fileinfo/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\;extension=openssl/s/^.*$/extension=openssl/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\;extension=pdo_mysql/s/^.*$/extension=pdd_mysql/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)date.timezone =/s/^.*$/date.timezone = Europe\/London/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)opcache.enable=/s/^.*$/opcache.enable=1/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)opcache.memory_consumption=/s/^.*$/opcache.memory_consumption=128/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)opcache.max_accelerated_files=/s/^.*$/opcache.max_accelerated_files=10000/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)opcache.revalidate_freq=/s/^.*$/opcache.revalidate_freq=2/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)opcache.save_comments=/s/^.*$/opcache.save_comments=1/' /etc/php/$phpver/fpm/php.ini
sudo systemctl restart php$phpver-fpm

# Check if OpenResty is installed and install it if not
if ! command -v openresty &> /dev/null
then
 # find version of Ubuntu we are using
 uver=$(lsb_release -sr)
 uarch=$(dpkg --print-architecture)

 cd ~
 
if [[ $uarch == "amd64" ]];
then
  echo amd64
  if [[ $uver < 22 ]]
  then
    echo Ubuntu 20
    wget -O - https://openresty.org/package/pubkey.gpg | sudo apt-key add -
    echo "deb http://openresty.org/package/ubuntu $(lsb_release -sc) main" | sudo tee /etc/apt/sources.list.d/openresty.list
  else
    echo Ubuntu 22
    wget -O - https://openresty.org/package/pubkey.gpg | sudo gpg --dearmor -o /usr/share/keyrings/openresty.gpg
    echo "deb [arch=$(dpkg --print-architecture) signed-by=/usr/share/keyrings/openresty.gpg] http://openresty.org/package/ubuntu $(lsb_release -sc) main" | sudo tee /etc/apt/sources.list.d/openresty.list > /dev/null
  fi
else
  if [[ $uver < 22 ]]
  then
    echo Ubuntu 20
    wget -O - https://openresty.org/package/pubkey.gpg | sudo apt-key add -
    echo "deb http://openresty.org/package/arm64/ubuntu $(lsb_release -sc) main" | sudo tee /etc/apt/sources.list.d/openresty.list
  else
    echo Ubuntu 22
    wget -O - https://openresty.org/package/pubkey.gpg | sudo gpg --dearmor -o /usr/share/keyrings/openresty.gpg
     echo "deb [arch=$(dpkg --print-architecture) signed-by=/usr/share/keyrings/openresty.gpg] http://openresty.org/package/arm64/ubuntu $(lsb_release -sc) main" | sudo tee /etc/apt/sources.list.d/openresty.list > /dev/null
  fi
fi

sudo apt update
sudo apt install openresty openresty-resty openresty-restydoc openresty-opm openresty-zlib openresty-pcre luarocks -y
sudo systemctl enable openresty
sudo systemctl start openresty

sudo mv /usr/local/openresty/nginx/conf/nginx.{conf,original}
sudo mv nginx.conf /usr/local/openresty/nginx/conf/nginx.conf

sudo mkdir /usr/local/openresty/nginx/sites
sudo mkdir /var/log/openresty
echo 'fastcgi_param HTTP_PROXY "";' | sudo tee -a /usr/local/openresty/nginx/conf/fastcgi.conf
echo 'fastcgi_param HTTP_PROXY "";' | sudo tee -a /usr/local/openresty/nginx/conf/fastcgi_params

sudo mv default.conf /usr/local/openresty/nginx/sites/default.conf

sudo mkdir /usr/local/openresty/nginx/html/default
echo "<html><head></head><body</body></html>" | sudo tee /usr/local/openresty/nginx/html/default/index.html > /dev/null

sudo systemctl reload openresty
fi

# Set up new OpenResty site
echo "server {" | sudo tee /usr/local/openresty/nginx/sites/$sitename.conf > /dev/null
echo "  listen 80;" | sudo tee -a /usr/local/openresty/nginx/sites/$sitename.conf > /dev/null
echo "  listen [::]:80;" | sudo tee -a /usr/local/openresty/nginx/sites/$sitename.conf > /dev/null
echo "  server_name $sitename www.$sitename;" | sudo tee -a /usr/local/openresty/nginx/sites/$sitename.conf > /dev/null
echo "  root /usr/local/openresty/nginx/html/$sitename;" | sudo tee -a /usr/local/openresty/nginx/sites/$sitename.conf > /dev/null
echo "}" | sudo tee -a /usr/local/openresty/nginx/sites/$sitename.conf > /dev/null
sudo mkdir /usr/local/openresty/nginx/html/$sitename
sudo chown -R www-data:www-data /usr/local/openresty/nginx/html/$sitename
sudo chmod -R 755 /usr/local/openresty/nginx/html/$sitename
sudo systemctl reload openresty

# Create a TLS certificate for the site
if ! command -v certbot &> /dev/null
then
  sudo apt-get install snapd
  sudo snap install core; sudo snap refresh core
  sudo snap install --classic certbot
  sudo ln -s /snap/bin/certbot /usr/bin/certbot
fi

sudo certbot certonly --webroot -w /usr/local/openresty/nginx/html/$sitename -d $sitename -d www.$sitename

# Add final version of server block
sudo rm /usr/local/openresty/nginx/sites/$sitename.conf
sed -i "s/domain/$sitename/g" serverblock.conf
sed -i "s/phpver/php$phpver/g" serverblock.conf
sudo mv serverblock.conf /usr/local/openresty/nginx/sites/$sitename.conf
openssl dhparam 2048 | sudo tee /usr/local/openresty/nginx/$sitename.dhparam
sudo systemctl reload openresty

# Check if MariaDB is installed and install it if not
if ! command -v mysql &> /dev/null
then
  sudo apt install mariadb-server -y
  sudo mysql_secure_installation
  sudo sed -i -e '/^\(;\|\)bind-address            = 127.0.0.1/s/^.*$/bind-address            = 127.0.0.1\nlocal_infile = 0/' /etc/mysql/mariadb.conf.d/50-server.cnf
fi

# Create database
databasename=$(openssl rand -hex 2)
databaseuser=$(openssl rand -hex 2)
databasepass=$(openssl rand -hex 8)

sudo mysql -e "CREATE DATABASE $databasename CHARACTER SET utf8mb4"
sudo mysql -e "CREATE USER '$databaseuser'@'localhost' IDENTIFIED BY '$databasepass'"
sudo mysql -e "GRANT ALL PRIVILEGES ON $databasename.* TO '$databaseuser'@'localhost'"
sudo mysql -e "FLUSH PRIVILEGES"

# Install ModX
cd ~
wget https://modx.com/download/direct/modx-3.0.3-pl.zip
unzip modx-3.0.3-pl.zip
cd modx-3.0.3-pl
sudo cp -r * /usr/local/openresty/nginx/html/$sitename
cd ..
rm -rf modx-3.0.3-pl modx-3.0.3-pl.zip
sudo touch /usr/local/openresty/nginx/html/$sitename/error.html
sudo touch /usr/local/openresty/nginx/html/$sitename/core/config/config.inc.php
sudo chown -R www-data:www-data /usr/local/openresty/nginx/html/$sitename
sudo chmod -R 755 /usr/local/openresty/nginx/html/$sitename

# Finished
echo.
echo Installation complete
echo
echo You should now visit $sitename/setup in your browser to finish the installation
echo
echo You will need, and should make a record of, the following details
echo
echo Site name: $sitename
echo Site management portal: $sitename/manager
echo
echo Database name    : $databasename
echo Database user    : $databaseuser
echo Database password: $databasepass
echo
echo NOTE: If you have installed ModX on an IPv6 only server you will need to set up a gateway
echo       before you will be able to download any ModX extras.
echo